@@ -24,6 +24,9 @@ from gitlabform.processors.project.project_push_rules_processor import (
 from gitlabform.processors.project.project_settings_processor import (
     ProjectSettingsProcessor,
 )
+from gitlabform.processors.shared.protected_environments_processor import (
+    ProtectedEnvironmentsProcessor,
+)
 from gitlabform.processors.project.resource_groups_processor import (
     ResourceGroupsProcessor,
 )
@@ -58,4 +61,5 @@ class ProjectProcessors(AbstractProcessors):
             SchedulesProcessor(gitlab),
             BadgesProcessor(gitlab),
             ResourceGroupsProcessor(gitlab),
+            ProtectedEnvironmentsProcessor(gitlab),
         ]
