@@ -11,7 +11,7 @@ from jinja2 import Environment, FileSystemLoader
 from gitlabform import EXIT_INVALID_INPUT
 from gitlabform.configuration import Configuration
 from gitlabform.gitlab import GitLab
-from gitlabform.gitlab.core import NotFoundException
+from gitlabform.gitlab.core import NotFoundException, UnexpectedResponseException
 from gitlabform.processors.abstract_processor import AbstractProcessor
 from gitlabform.processors.util.branch_protector import BranchProtector
 
@@ -67,24 +67,16 @@ class FilesProcessor(AbstractProcessor):
                         exit_code=EXIT_INVALID_INPUT,
                     )
 
-                # unprotect protected branch temporarily for operations below
-                if configuration.get("branches|" + branch + "|protected"):
-                    debug(
-                        "> Temporarily unprotecting the branch for managing files in it..."
-                    )
-                    self.branch_protector.unprotect_branch(project_and_group, branch)
-
                 if configuration.get("files|" + file + "|delete"):
                     try:
                         self.gitlab.get_file(project_and_group, branch, file)
                         debug("Deleting file '%s' in branch '%s'", file, branch)
-                        self.gitlab.delete_file(
+                        self.modify_file_dealing_with_branch_protection(
                             project_and_group,
                             branch,
                             file,
-                            self.get_commit_message_for_file_change(
-                                "delete", file, configuration
-                            ),
+                            "delete",
+                            configuration,
                         )
                     except NotFoundException:
                         debug(
@@ -99,19 +91,18 @@ class FilesProcessor(AbstractProcessor):
                         new_content = configuration.get("files|" + file + "|content")
                     else:
                         path_in_config = Path(
-                            configuration.get("files|" + file + "|file")
+                            str(configuration.get("files|" + file + "|file"))
                         )
                         if path_in_config.is_absolute():
-                            # TODO: does this work? we are reading the content twice in this case...
-                            path = path_in_config.read_text()
+                            effective_path = path_in_config
                         else:
                             # relative paths are relative to config file location
-                            path = Path(
+                            effective_path = Path(
                                 os.path.join(
                                     self.config.config_dir, str(path_in_config)
                                 )
                             )
-                        new_content = path.read_text()
+                        new_content = effective_path.read_text()
 
                     if configuration.get("files|" + file + "|template", True):
                         new_content = self.get_file_content_as_template(
@@ -127,14 +118,13 @@ class FilesProcessor(AbstractProcessor):
                         if current_content != new_content:
                             if configuration.get("files|" + file + "|overwrite"):
                                 debug("Changing file '%s' in branch '%s'", file, branch)
-                                self.gitlab.set_file(
+                                self.modify_file_dealing_with_branch_protection(
                                     project_and_group,
                                     branch,
                                     file,
+                                    "modify",
+                                    configuration,
                                     new_content,
-                                    self.get_commit_message_for_file_change(
-                                        "change", file, configuration
-                                    ),
                                 )
                             else:
                                 debug(
@@ -151,26 +141,115 @@ class FilesProcessor(AbstractProcessor):
                             )
                     except NotFoundException:
                         debug("Creating file '%s' in branch '%s'", file, branch)
-                        self.gitlab.add_file(
+                        self.modify_file_dealing_with_branch_protection(
                             project_and_group,
                             branch,
                             file,
+                            "add",
+                            configuration,
                             new_content,
-                            self.get_commit_message_for_file_change(
-                                "add", file, configuration
-                            ),
                         )
 
-                # protect branch back after above operations
-                if configuration.get("branches|" + branch + "|protected"):
-                    debug("> Protecting the branch again.")
-                    self.branch_protector.protect_branch(
-                        project_and_group, configuration, branch
-                    )
                 if configuration.get("files|" + file + "|only_first_branch"):
                     verbose("Skipping other branches for this file, as configured.")
                     break
 
+    def modify_file_dealing_with_branch_protection(
+        self,
+        project_and_group,
+        branch,
+        file,
+        operation,
+        configuration,
+        new_content=None,
+    ):
+        # perhaps your user permissions are ok to just perform this operation regardless
+        # of the branch protection...
+
+        try:
+
+            self.just_modify_file(
+                project_and_group, branch, file, operation, configuration, new_content
+            )
+
+        except UnexpectedResponseException as e:
+
+            if (
+                e.response_status_code == 400
+                and "You are not allowed to push into this branch" in e.response_text
+            ):
+
+                # ...but if not, then we can unprotect the branch, but only if we know how to
+                # protect it again...
+
+                if configuration.get("branches|" + branch + "|protected"):
+                    debug(
+                        f"> Temporarily unprotecting the branch to {operation} a file in it..."
+                    )
+                    self.branch_protector.unprotect_branch(project_and_group, branch)
+                else:
+                    fatal(
+                        f"Operation {operation} on file {file} in branch {branch} not permitted,"
+                        f" but we don't have a branch protection configuration provided for this"
+                        f" branch. Breaking as we cannot unprotect the branch as we would not know"
+                        f" how to protect it again.",
+                        EXIT_INVALID_INPUT,
+                    )
+
+                try:
+                    self.just_modify_file(
+                        project_and_group,
+                        branch,
+                        file,
+                        operation,
+                        configuration,
+                        new_content,
+                    )
+
+                finally:
+                    # ...and protect the branch again after the operation
+                    if configuration.get("branches|" + branch + "|protected"):
+                        debug("> Protecting the branch again.")
+                        self.branch_protector.protect_branch(
+                            project_and_group, configuration, branch
+                        )
+
+            else:
+                raise e
+
+    def just_modify_file(
+        self,
+        project_and_group,
+        branch,
+        file,
+        operation,
+        configuration,
+        new_content=None,
+    ):
+        if operation == "modify":
+            self.gitlab.set_file(
+                project_and_group,
+                branch,
+                file,
+                new_content,
+                self.get_commit_message_for_file_change("change", file, configuration),
+            )
+        elif operation == "add":
+            self.gitlab.add_file(
+                project_and_group,
+                branch,
+                file,
+                new_content,
+                self.get_commit_message_for_file_change("add", file, configuration),
+            )
+        elif operation == "delete":
+            self.gitlab.delete_file(
+                project_and_group,
+                branch,
+                file,
+                self.get_commit_message_for_file_change("delete", file, configuration),
+            )
+
     def get_file_content_as_template(self, template, project_and_group, **kwargs):
         # Use jinja with variables project and group
         rtemplate = Environment(
